import { useLocation } from "react-router-dom";
import { useState } from "react";
import clsx from "clsx";
import FormBox from "../components/FormBox";
import Button from "../components/Button";
import { useToast } from "../hooks/useToast";
import { lockerService } from "../services/locker.service";
import { Copy } from "lucide-react";
import { AnimatePresence } from "motion/react";
import { motion } from "motion/react";
import { useLoading } from "../hooks/useLoading";

function LockersRegistration() {
  const location = useLocation();
  const showToast = useToast();
  const { setIsLoading } = useLoading();
  const { noOfLockers, cabinetId } = location.state;
  const [submitting, setSubmitting] = useState(false);
  const [notification, setNotification] = useState({
    visible: false,
    message: "",
    type: "",
  });

  const initLockers = Array(parseInt(noOfLockers))
    .fill()
    .map(() => ({
      id: "",
      size: "",
      keyId: "",
      idValid: true,
      sizeValid: true,
      keyIdValid: true,
    }));
  const [lockerValues, setLockerValues] = useState(initLockers);
  if(!location.state) {
    return <></>
  }
  const handleSubmit = async (e) => {
    console.log("submitting");
    e.preventDefault();
    const idRegex = /^[A-Z]{2}[0-9]{4}$/;
    let valid = true;

    // Helper function to find duplicates
    const findDuplicates = (arr, key) => {
      const values = arr.map((item) => item[key]);
      return values.filter((item, index) => values.indexOf(item) !== index);
    };

    // Find duplicates
    const duplicateLockerIds = findDuplicates(lockerValues, "id");
    const duplicateKeyIds = findDuplicates(lockerValues, "keyId");

    const newValues = lockerValues.map((locker) => {
      const newLocker = { ...locker };

      // Check ID
      if (
        locker.id === "" ||
        !idRegex.test(locker.id) ||
        duplicateLockerIds.includes(locker.id)
      ) {
        newLocker.idValid = false;
        valid = false;
      } else {
        newLocker.idValid = true;
      }

      // Check size
      if (locker.size === "") {
        newLocker.sizeValid = false;
        valid = false;
      } else {
        newLocker.sizeValid = true;
      }

      // Check keyId
      if (
        locker.keyId === "" ||
        !idRegex.test(locker.keyId) ||
        duplicateKeyIds.includes(locker.keyId)
      ) {
        newLocker.keyIdValid = false;
        valid = false;
      } else {
        newLocker.keyIdValid = true;
      }

      return newLocker;
    });

    setLockerValues(newValues);

    if (!valid) {
      const errorMessage =
        duplicateLockerIds.length || duplicateKeyIds.length
          ? "Please ensure all IDs are unique."
          : "Inavlid Ids";
      showToast(errorMessage, "error");
      return;
    }

    try {
      setSubmitting(true);
      setIsLoading(true);
      const response = await lockerService.registerLockers(
        cabinetId,
        lockerValues
      );
      setNotification({
        visible: true,
        message: `Cabinet creation successful. Cabinet ID: ${response.data.cabinetId}`,
        type: "success",
      });
    } catch (error) {
      console.error(error);
      setNotification({
        visible: true,
        message: `Error registering lockers. ${error.message}`,
        type: "error",
      });
      return;
    } finally {
      setIsLoading(false);
      setSubmitting(false);
    }
  };

  const lockerDetails = lockerValues.map((locker, index) => {
    return (
      <div key={index} className="flex gap-12 items-center">
        <label className="text-lg text-bold dark:text-primary-dark">{`Locker ${
          index + 1
        }`}</label>

        <input
          value={locker.id}
          disabled={submitting}
          className={clsx(
            "w-64 h-9 px-2 rounded-full dark:bg-white dark:text-grey border-2 text-grey focus:outline-grey",
            !locker.idValid && "border-error text-error"
          )}
          onChange={(e) => {
            const newValues = [...lockerValues];
            newValues[index] = {
              ...newValues[index],
              id: e.target.value.toUpperCase(),
              idValid: true,
            };
            setLockerValues(newValues);
          }}
          placeholder="Locker ID"
          type="text"
          maxLength={6}
        />

        <select
          value={locker.size}
          disabled={submitting}
          className={clsx(
            "w-64 h-9 px-2 rounded-full dark:bg-white dark:text-grey border-2 text-grey focus:outline-grey",
            !locker.sizeValid && "border-error"
          )}
          onChange={(e) => {
            const newValues = [...lockerValues];
            newValues[index] = {
              ...newValues[index],
              size: e.target.value,
              sizeValid: true,
            };
            setLockerValues(newValues);
          }}
        >
          <option value="" disabled>
            Size
          </option>
          <option value="small">Small</option>
          <option value="medium">Medium</option>
          <option value="large">Large</option>
        </select>

        <input
          value={locker.keyId}
          disabled={submitting}
          className={clsx(
            "w-64 h-9 px-2 rounded-full dark:bg-white dark:text-grey border-2 text-grey focus:outline-grey",
            !locker.keyIdValid && "border-error"
          )}
          onChange={(e) => {
            const newValues = [...lockerValues];
            newValues[index] = {
              ...newValues[index],
              keyId: e.target.value.toUpperCase(),
              keyIdValid: true,
            };
            setLockerValues(newValues);
          }}
          placeholder="Key ID"
          type="text"
          maxLength={6}
        />
      </div>
    );
  });

  return (
    <div>
      <AnimatePresence>
        {notification.visible && (
          <motion.div
            initial={{ opacity: 0 }}
            animate={{ opacity: 1 }}
            exit={{ opacity: 0 }}
            className={clsx(
              "p-4 mb-8 font-body text-center text-xl rounded-2xl flex items-center justify-center gap-2",
              notification.type === "error"
                ? "bg-error-surface text-error"
                : "bg-success-surface text-success"
            )}
          >
            {notification.type === "error" ? notification.message : notification.message.split(":").map((msg, index) => {
              return index === 1 ? (
                <span key={index} className="border-b border-dashed">
                  {msg}
                </span>
              ) : (
                <span key={index}>{msg}</span>
              );
            })}
            <Copy
              cursor={"pointer"}
              size={15}
              onClick={navigator.clipboard.writeText(
                notification.message.split(":")[1].trim()
              )}
            />
          </motion.div>
        )}
      </AnimatePresence>
      <div className="relative">
        {notification.type === "success" && (
          <div className="absolute inset-0 bg-[#fff]/50 z-10 rounded-3xl" />
        )}
        <FormBox title="Locker Registration" disabled={submitting}>
          <div className="px-4 pt-7 text-2xl font-bold text-primary dark:text-primary-dark">
            {cabinetId}
          </div>
          <div className="flex flex-col gap-4 p-4">{lockerDetails}</div>
          <Button
            disabled={submitting}
            text="Register"
            onClick={(e) => {
              handleSubmit(e);
            }}
          />
        </FormBox>
      </div>
    </div>
  );
}

export default LockersRegistration;
